@@ -1828,6 +1828,42 @@ package Sidef::Types::Array::Array {
 
     *each_comb = \&combinations;
 
+    sub nth_permutation {
+        my ($self, $n) = @_;
+
+        my @perm;
+        my @arr = @$self;
+
+        $n = $n->int;
+
+        my $cmp = CORE::int($n->cmp(Sidef::Types::Number::Number::ZERO));
+
+        if ($cmp < 0) {
+            $n   = $n->neg->dec;
+            @arr = CORE::reverse(@arr);
+        }
+        elsif ($cmp == 0) {
+            return bless \@arr, __PACKAGE__;
+        }
+        else {
+            $n = $n->dec;
+        }
+
+        while (@arr) {
+            my $end = $#arr;
+            my $f   = Sidef::Types::Number::Number->_set_uint($end)->factorial;
+            (my $q, $n) = $n->divmod($f);
+            $q = $q->imod(Sidef::Types::Number::Number->_set_uint($end + 1));
+            my $i = CORE::int($q);
+            push @perm, $arr[$i];
+            @arr = (@arr[0 .. $i - 1, $i + 1 .. $end]);
+        }
+
+        bless \@perm, __PACKAGE__;
+    }
+
+    *nth_perm = \&nth_permutation;
+
     sub permutations {
         my ($self, $code) = @_;
 
@@ -1906,7 +1942,7 @@ package Sidef::Types::Array::Array {
         my @indices = (0) x @arrs;
         my (@temp, @cartesian);
 
-      OUTER: while ($more) {
+        while ($more) {
             @temp = @indices;
 
             for (my $i = $#indices ; $i >= 0 ; --$i) {
@@ -1921,10 +1957,10 @@ package Sidef::Types::Array::Array {
             }
 
             if (defined($block)) {
-                $block->run(map { @$_ ? $_->[CORE::shift(@temp)] : () } @arrs);
+                $block->run(map { $_->[CORE::shift(@temp)] } @arrs);
             }
             else {
-                push @cartesian, bless [map { @$_ ? $_->[CORE::shift(@temp)] : () } @arrs], __PACKAGE__;
+                push @cartesian, bless([map { $_->[CORE::shift(@temp)] } @arrs], __PACKAGE__);
             }
         }
 
